##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test add-form"""
import unittest

from zope.browser.interfaces import IAdding
from zope.component import getMultiAdapter
from zope.component.eventtesting import PlacelessSetup as EventPlacelessSetup
from zope.component.eventtesting import getEvents
from zope.component.factory import Factory
from zope.component.interfaces import IFactory
from zope.component.testing import PlacelessSetup as CAPlacelessSetup
from zope.formlib.widget import CustomWidgetFactory
from zope.interface import Interface
from zope.interface import implementer
from zope.interface.interfaces import IComponentLookup
from zope.lifecycleevent.interfaces import IObjectCreatedEvent
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.publisher.browser import TestRequest
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.schema import TextLine
from zope.schema import accessors
from zope.security.checker import CheckerPublic
from zope.site.site import SiteManagerAdapter

import zope.app.form.testing as ztapi
from zope.app.form.browser import TextWidget as Text
from zope.app.form.browser.add import AddView
from zope.app.form.browser.add import AddViewFactory
from zope.app.form.browser.metaconfigure import AddFormDirective
from zope.app.form.browser.submit import Update
# Foo needs to be imported as globals() are checked
from zope.app.form.browser.tests.test_editview import Foo
from zope.app.form.browser.tests.test_editview import FooBarAdapter
from zope.app.form.browser.tests.test_editview import IBar
from zope.app.form.browser.tests.test_editview import IFoo


class PlacelessSetup(CAPlacelessSetup, EventPlacelessSetup):

    def setUp(self, doctesttest=None):
        CAPlacelessSetup.setUp(self)
        EventPlacelessSetup.setUp(self)


class Context:

    def action(self, discriminator, callable, args=(), kw={}):
        self.last_action = (discriminator, callable, args, kw)


class ITest(Interface):

    name = TextLine()
    first = TextLine()
    last = TextLine()
    email = TextLine()
    address = TextLine()
    getfoo, setfoo = accessors(TextLine())
    extra1 = TextLine()
    extra2 = TextLine(required=False)


@implementer(ITest)
class C:

    def __init__(self, *args, **kw):
        self.args = args
        self.kw = kw

    def getfoo(self): return self._foo
    def setfoo(self, v): self._foo = v


class V:
    name_widget = CustomWidgetFactory(Text)
    first_widget = CustomWidgetFactory(Text)
    last_widget = CustomWidgetFactory(Text)
    email_widget = CustomWidgetFactory(Text)
    address_widget = CustomWidgetFactory(Text)
    getfoo_widget = CustomWidgetFactory(Text)
    extra1_widget = CustomWidgetFactory(Text)
    extra2_widget = CustomWidgetFactory(Text)


class FooV:
    bar_widget = CustomWidgetFactory(Text)


class SampleData:

    name = "foo"
    first = "bar"
    last = "baz"
    email = "baz@dot.com"
    address = "aa"
    getfoo = "foo"
    extra1 = "extra1"
    extra2 = "extra2"


class Test(PlacelessSetup, unittest.TestCase):

    def setUp(self):
        self._context = Context()
        super().setUp()
        ztapi.provideAdapter(IFoo, IBar, FooBarAdapter)

    def _invoke_add(self, schema=ITest, name="addthis",
                    permission="zope.Public",
                    label="Add this", content_factory=C, class_=V,
                    arguments=['first', 'last'], keyword_arguments=['email'],
                    set_before_add=['getfoo'], set_after_add=['extra1'],
                    fields=None):
        """Call the 'add' factory to process arguments into 'args'."""
        AddFormDirective(self._context,
                         schema=schema,
                         name=name,
                         permission=permission,
                         label=label,
                         content_factory=content_factory,
                         class_=class_,
                         arguments=arguments,
                         keyword_arguments=keyword_arguments,
                         set_before_add=set_before_add,
                         set_after_add=set_after_add,
                         fields=fields
                         )()

    def test_add_no_fields(self):
        _context = self._context
        self._invoke_add()
        result1 = _context.last_action
        self._invoke_add(
            fields=[
                "name",
                "first",
                "last",
                "email",
                "address",
                "getfoo",
                "extra1",
                "extra2",
            ]
        )
        result2 = _context.last_action

        self.assertEqual(result1, result2)

    def test_add_error_handling(self):
        # cannot use a field in arguments if it is not mentioned in fields
        self.assertRaises(ValueError, self._invoke_add,
                          fields="first email getfoo extra1".split())

        # cannot use a field in keyword_arguments if it is not
        # mentioned in fields

        self.assertRaises(ValueError, self._invoke_add,
                          fields="first last getfoo extra1".split())

        # cannot use a field in set_before_add if it is not mentioned in fields
        self.assertRaises(ValueError, self._invoke_add,
                          fields="first last email extra1".split())

        # cannot use a field in set_after_add if it is not mentioned in fields
        self.assertRaises(ValueError, self._invoke_add,
                          fields="first last email getfoo".split())

        # cannot use an optional field in arguments
        self.assertRaises(ValueError, self._invoke_add, arguments=["extra2"])

    def test_add(self, args=None):
        self._invoke_add()
        (descriminator, callable, args, kw) = self._context.last_action

        self.assertEqual(descriminator,
                         ('view', IAdding, 'addthis', IBrowserRequest,
                          IDefaultBrowserLayer))

        self.assertEqual(callable, AddViewFactory)

        (name, schema, label, permission, layer, template,
         default_template, bases, for_, fields, content_factory,
         arguments, keyword_arguments, set_before_add,
         set_after_add) = args

        self.assertEqual(name, 'addthis')
        self.assertEqual(schema, ITest)
        self.assertEqual(label, 'Add this')
        self.assertEqual(permission, CheckerPublic)  # 'zope.Public' translated
        self.assertEqual(layer, IDefaultBrowserLayer)
        self.assertEqual(template, 'add.pt')
        self.assertEqual(default_template, 'add.pt')
        self.assertEqual(bases, (V, AddView, ))
        self.assertEqual(for_, IAdding)
        self.assertEqual(" ".join(fields),
                         "name first last email address getfoo extra1 extra2")
        self.assertEqual(content_factory, C)
        self.assertEqual(" ".join(arguments),
                         "first last")
        self.assertEqual(" ".join(keyword_arguments),
                         "email")
        self.assertEqual(" ".join(set_before_add),
                         "getfoo")
        self.assertEqual(" ".join(set_after_add),
                         "extra1 name address extra2")

    def test_add_content_factory_id(self, args=None):
        self._invoke_add(content_factory='C')
        (descriminator, callable, args, kw) = self._context.last_action

        self.assertEqual(descriminator,
                         ('view', IAdding, 'addthis', IBrowserRequest,
                          IDefaultBrowserLayer))

        self.assertEqual(callable, AddViewFactory)

        (name, schema, label, permission, layer, template,
         default_template, bases, for_, fields, content_factory,
         arguments, keyword_arguments, set_before_add,
         set_after_add) = args

        self.assertEqual(name, 'addthis')
        self.assertEqual(schema, ITest)
        self.assertEqual(label, 'Add this')
        self.assertEqual(permission, CheckerPublic)  # 'zope.Public' translated
        self.assertEqual(layer, IDefaultBrowserLayer)
        self.assertEqual(template, 'add.pt')
        self.assertEqual(default_template, 'add.pt')
        self.assertEqual(bases, (V, AddView, ))
        self.assertEqual(for_, IAdding)
        self.assertEqual(" ".join(fields),
                         "name first last email address getfoo extra1 extra2")
        self.assertEqual(content_factory, 'C')
        self.assertEqual(" ".join(arguments),
                         "first last")
        self.assertEqual(" ".join(keyword_arguments),
                         "email")
        self.assertEqual(" ".join(set_before_add),
                         "getfoo")
        self.assertEqual(" ".join(set_after_add),
                         "extra1 name address extra2")

    def test_create(self):

        @implementer(IAdding)
        class Adding:

            def __init__(self, test):
                self.test = test

            def add(self, ob):
                self.ob = ob
                self.test.assertEqual(
                    ob.__dict__,
                    {'args': ("bar", "baz"),
                     'kw': {'email': 'baz@dot.com'},
                     '_foo': 'foo',
                     })
                return ob

            def nextURL(self):
                return "."

        adding = Adding(self)
        self._invoke_add()
        (descriminator, callable, args, kw) = self._context.last_action
        AddViewFactory(*args)
        request = TestRequest()
        view = getMultiAdapter((adding, request), name='addthis')
        content = view.create('a', 0, abc='def')

        self.assertIsInstance(content, C)
        self.assertEqual(content.args, ('a', 0))
        self.assertEqual(content.kw, {'abc': 'def'})

    def test_create_content_factory_id(self):

        @implementer(IAdding)
        class Adding:

            def __init__(self, test):
                self.test = test

            def add(self, ob):
                self.ob = ob
                self.test.assertEqual(
                    ob.__dict__,
                    {'args': ("bar", "baz"),
                     'kw': {'email': 'baz@dot.com'},
                     '_foo': 'foo',
                     })
                return ob

            def nextURL(self):
                return "."

        # register content factory for content factory id lookup
        ztapi.provideAdapter(None, IComponentLookup, SiteManagerAdapter)
        ztapi.provideUtility(IFactory, Factory(C), name='C')

        adding = Adding(self)
        self._invoke_add(content_factory='C')
        (descriminator, callable, args, kw) = self._context.last_action
        AddViewFactory(*args)
        request = TestRequest()
        view = getMultiAdapter((adding, request), name='addthis')
        content = view.create('a', 0, abc='def')

        self.assertIsInstance(content, C)
        self.assertEqual(content.args, ('a', 0))
        self.assertEqual(content.kw, {'abc': 'def'})

    def test_createAndAdd(self):

        @implementer(IAdding)
        class Adding:

            def __init__(self, test):
                self.test = test

            def add(self, ob):
                self.ob = ob
                self.test.assertEqual(
                    ob.__dict__,
                    {'args': ("bar", "baz"),
                     'kw': {'email': 'baz@dot.com'},
                     '_foo': 'foo',
                     })
                return ob

            def nextURL(self):
                return "."

        adding = Adding(self)
        self._invoke_add()
        (descriminator, callable, args, kw) = self._context.last_action
        AddViewFactory(*args)
        request = TestRequest()
        view = getMultiAdapter((adding, request), name='addthis')

        view.createAndAdd(SampleData.__dict__)

        self.assertEqual(adding.ob.extra1, "extra1")
        self.assertEqual(adding.ob.extra2, "extra2")
        self.assertEqual(adding.ob.name, "foo")
        self.assertEqual(adding.ob.address, "aa")
        self.assertEqual(len(getEvents(IObjectCreatedEvent)), 1)
        self.assertEqual(len(getEvents(IObjectModifiedEvent)), 1)

    def test_createAndAdd_w_adapter(self):

        @implementer(IAdding)
        class Adding:

            def __init__(self, test):
                self.test = test

            def add(self, ob):
                self.ob = ob
                self.test.assertEqual(ob.__dict__, {'foo': 'bar'})
                return ob

            def nextURL(self):
                return "."

        adding = Adding(self)
        self._invoke_add(
            schema=IBar, name="addthis", permission="zope.Public",
            label="Add this", content_factory=Foo, class_=FooV,
            arguments=None, keyword_arguments=None,
            set_before_add=["bar"], set_after_add=None,
            fields=None
        )
        (descriminator, callable, args, kw) = self._context.last_action
        AddViewFactory(*args)
        request = TestRequest()
        view = getMultiAdapter((adding, request), name='addthis')

        view.createAndAdd({'bar': 'bar'})

    def test_hooks(self):

        @implementer(IAdding)
        class Adding:
            pass

        adding = Adding()
        self._invoke_add()
        (descriminator, callable, args, kw) = self._context.last_action
        AddViewFactory(*args)
        request = TestRequest()

        request.form.update({
            "field.%s" % k: v
            for (k, v) in dict(SampleData.__dict__).items()
        })
        request.form[Update] = ''
        view = getMultiAdapter((adding, request), name='addthis')

        # Add hooks to V

        l_ = [None]

        def add(aself, ob):
            l_[0] = ob
            self.assertEqual(
                ob.__dict__,
                {'args': ("bar", "baz"),
                 'kw': {'email': 'baz@dot.com'},
                 '_foo': 'foo',
                 })
            return ob

        V.add = add

        V.nextURL = lambda self: 'next'

        try:
            self.assertEqual(view.update(), '')

            self.assertEqual(view.errors, ())

            self.assertEqual(l_[0].extra1, "extra1")
            self.assertEqual(l_[0].extra2, "extra2")
            self.assertEqual(l_[0].name, "foo")
            self.assertEqual(l_[0].address, "aa")

            self.assertEqual(request.response.getHeader("Location"), "next")

            # Verify that calling update again doesn't do anything.
            l_[0] = None
            self.assertEqual(view.update(), '')
            self.assertEqual(l_[0], None)

        finally:
            # Uninstall hooks
            del V.add
            del V.nextURL


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromTestCase(Test)



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser widgets"""
__docformat__ = 'restructuredtext'

# the implementation of widgets has moved to zope.formlib.widgets
# import directly from there instead.

from zope.formlib.widget import BrowserWidget
from zope.formlib.widget import DisplayWidget
from zope.formlib.widget import UnicodeDisplayWidget
from zope.formlib.widgets import ASCIIAreaWidget
from zope.formlib.widgets import ASCIIDisplayWidget
from zope.formlib.widgets import ASCIIWidget
from zope.formlib.widgets import BooleanDropdownWidget
from zope.formlib.widgets import BooleanRadioWidget
from zope.formlib.widgets import BooleanSelectWidget
from zope.formlib.widgets import BytesAreaWidget
from zope.formlib.widgets import BytesDisplayWidget
from zope.formlib.widgets import BytesWidget
from zope.formlib.widgets import CheckBoxWidget
from zope.formlib.widgets import ChoiceCollectionDisplayWidget
from zope.formlib.widgets import ChoiceCollectionInputWidget
from zope.formlib.widgets import ChoiceDisplayWidget
from zope.formlib.widgets import ChoiceInputWidget
from zope.formlib.widgets import CollectionDisplayWidget
from zope.formlib.widgets import CollectionInputWidget
from zope.formlib.widgets import DateDisplayWidget
from zope.formlib.widgets import DateI18nWidget
from zope.formlib.widgets import DatetimeDisplayWidget
from zope.formlib.widgets import DatetimeI18nWidget
from zope.formlib.widgets import DatetimeWidget
from zope.formlib.widgets import DateWidget
from zope.formlib.widgets import DecimalWidget
from zope.formlib.widgets import DropdownWidget
from zope.formlib.widgets import FileWidget
from zope.formlib.widgets import FloatWidget
from zope.formlib.widgets import IntWidget
from zope.formlib.widgets import ItemDisplayWidget
from zope.formlib.widgets import ItemsMultiDisplayWidget
from zope.formlib.widgets import ListDisplayWidget
from zope.formlib.widgets import ListSequenceWidget
from zope.formlib.widgets import MultiCheckBoxWidget
from zope.formlib.widgets import MultiSelectFrozenSetWidget
from zope.formlib.widgets import MultiSelectSetWidget
from zope.formlib.widgets import MultiSelectWidget
from zope.formlib.widgets import ObjectWidget
from zope.formlib.widgets import OrderedMultiSelectWidget
from zope.formlib.widgets import PasswordWidget
from zope.formlib.widgets import RadioWidget
from zope.formlib.widgets import SelectWidget
from zope.formlib.widgets import SequenceDisplayWidget
from zope.formlib.widgets import SequenceWidget
from zope.formlib.widgets import SetDisplayWidget
from zope.formlib.widgets import TextAreaWidget
from zope.formlib.widgets import TextWidget
from zope.formlib.widgets import TupleSequenceWidget
from zope.formlib.widgets import URIDisplayWidget


__all__ = [
    'BrowserWidget',
    'DisplayWidget',
    'UnicodeDisplayWidget',
    'TextWidget',
    'BytesWidget',
    'TextAreaWidget',
    'BytesAreaWidget',
    'PasswordWidget',
    'FileWidget',
    'ASCIIWidget',
    'ASCIIAreaWidget',
    'IntWidget',
    'FloatWidget',
    'DecimalWidget',
    'DatetimeWidget',
    'DateWidget',
    'DatetimeI18nWidget',
    'DateI18nWidget',
    'DatetimeDisplayWidget',
    'DateDisplayWidget',
    'BytesDisplayWidget',
    'ASCIIDisplayWidget',
    'URIDisplayWidget',
    'CheckBoxWidget',
    'BooleanRadioWidget',
    'BooleanSelectWidget',
    'BooleanDropdownWidget',
    'ItemDisplayWidget',
    'ItemsMultiDisplayWidget',
    'SetDisplayWidget',
    'ListDisplayWidget',
    'ChoiceDisplayWidget',
    'ChoiceInputWidget',
    'CollectionDisplayWidget',
    'CollectionInputWidget',
    'ChoiceCollectionDisplayWidget',
    'ChoiceCollectionInputWidget',
    'SelectWidget',
    'DropdownWidget',
    'RadioWidget',
    'MultiSelectWidget',
    'MultiSelectSetWidget',
    'MultiSelectFrozenSetWidget',
    'MultiCheckBoxWidget',
    'OrderedMultiSelectWidget',
    'SequenceWidget',
    'TupleSequenceWidget',
    'ListSequenceWidget',
    'SequenceDisplayWidget',
    'ObjectWidget',
]



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Validation Exceptions

This has been moved to zope.formlib.interfaces.
"""


# this moved to zope.formlib.interfaces
from zope.formlib.interfaces import ConversionError
from zope.formlib.interfaces import ErrorContainer
from zope.formlib.interfaces import IDisplayWidget
from zope.formlib.interfaces import IInputWidget
from zope.formlib.interfaces import InputErrors
from zope.formlib.interfaces import IWidget
from zope.formlib.interfaces import IWidgetFactory
from zope.formlib.interfaces import IWidgetInputError
from zope.formlib.interfaces import MissingInputError
from zope.formlib.interfaces import WidgetInputError
from zope.formlib.interfaces import WidgetsError


__all__ = [
    'ConversionError',
    'ErrorContainer',
    'IDisplayWidget',
    'IInputWidget',
    'IWidget',
    'IWidgetFactory',
    'IWidgetInputError',
    'InputErrors',
    'MissingInputError',
    'WidgetInputError',
    'WidgetsError',
]



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Utilities for testing form machinery"""
import zope.security.checker
from zope.interface.interfaces import IMethod
from zope.schema import getFieldsInOrder
from zope.security.interfaces import Unauthorized


class DummyChecker:
    """a checker for testing that requires explicit declarations

    requires explicit declaration of what is and is not authorized; does not
    require testing machinery to set up an interaction or a request.

    To instantiate, pass two dictionaries, the first for get access attribute
    protection, and the second for set access attribute protection.  keys
    should be the attribute names, and values should be boolean True and
    False, where True indicates authorized and False, unauthorized.  Any
    attributes that are not explicitly set and, in the case of get protection,
    are not in the zope.security.checker._available_by_default list,
    will cause ForbiddenAttribute to be raised when the name is checked, as
    with the real zope.security checkers.
    """

    def __init__(self, getnames, setnames):
        self.getnames = getnames
        self.setnames = setnames

    def check_getattr(self, obj, name):
        if name not in zope.security.checker._available_by_default:
            val = self.getnames[name]
            if not val:
                raise Unauthorized
    check = check_getattr

    def check_setattr(self, obj, name):
        val = self.setnames[name]
        if not val:
            raise Unauthorized

    def proxy(self, value):
        return value


def SchemaChecker(schema, readonly=False):
    """returns a checker that allows read and write access to fields in schema.
    """
    get = {}
    set = {}
    for name, field in getFieldsInOrder(schema):
        get[name] = True
        if not field.readonly:
            if IMethod.providedBy(field):
                get[field.writer.__name__] = True
            else:
                set[name] = True
    assert not readonly
    return DummyChecker(get, set)


def securityWrap(ob, schema, readonly=False):
    return zope.security.checker.Proxy(ob, SchemaChecker(schema, readonly))



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser widgets with text-based input"""
# BBB implementation moved to zope.formlib.textwidgets
from zope.formlib.textwidgets import ASCII
from zope.formlib.textwidgets import ASCIIAreaWidget
from zope.formlib.textwidgets import ASCIIDisplayWidget
from zope.formlib.textwidgets import ASCIIWidget
from zope.formlib.textwidgets import Bytes
from zope.formlib.textwidgets import BytesAreaWidget
from zope.formlib.textwidgets import BytesDisplayWidget
from zope.formlib.textwidgets import BytesWidget
from zope.formlib.textwidgets import DateDisplayWidget
from zope.formlib.textwidgets import DateI18nWidget
from zope.formlib.textwidgets import DatetimeDisplayWidget
from zope.formlib.textwidgets import DatetimeI18nWidget
from zope.formlib.textwidgets import DatetimeWidget
from zope.formlib.textwidgets import DateWidget
from zope.formlib.textwidgets import DecimalWidget
from zope.formlib.textwidgets import FileWidget
from zope.formlib.textwidgets import FloatWidget
from zope.formlib.textwidgets import IntWidget
from zope.formlib.textwidgets import PasswordWidget
from zope.formlib.textwidgets import TextAreaWidget
from zope.formlib.textwidgets import TextWidget
from zope.formlib.textwidgets import URIDisplayWidget
from zope.formlib.textwidgets import escape


__all__ = [
    'ASCII',
    'ASCIIAreaWidget',
    'ASCIIDisplayWidget',
    'ASCIIWidget',
    'Bytes',
    'BytesAreaWidget',
    'BytesDisplayWidget',
    'BytesWidget',
    'DateDisplayWidget',
    'DateI18nWidget',
    'DateWidget',
    'DatetimeDisplayWidget',
    'DatetimeI18nWidget',
    'DatetimeWidget',
    'DecimalWidget',
    'FileWidget',
    'FloatWidget',
    'IntWidget',
    'PasswordWidget',
    'TextAreaWidget',
    'TextWidget',
    'URIDisplayWidget',
    'escape',
]



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Source widgets support"""
# BBB
from zope.formlib.source import IterableSourceVocabulary
from zope.formlib.source import SourceDisplayWidget
from zope.formlib.source import SourceDropdownWidget
from zope.formlib.source import SourceInputWidget
from zope.formlib.source import SourceListInputWidget
from zope.formlib.source import SourceMultiCheckBoxWidget
from zope.formlib.source import SourceMultiSelectFrozenSetWidget
from zope.formlib.source import SourceMultiSelectSetWidget
from zope.formlib.source import SourceMultiSelectWidget
from zope.formlib.source import SourceOrderedMultiSelectWidget
from zope.formlib.source import SourceRadioWidget
from zope.formlib.source import SourceSelectWidget
from zope.formlib.source import SourceSequenceDisplayWidget


__all__ = [
    'IterableSourceVocabulary',
    'SourceDisplayWidget',
    'SourceDropdownWidget',
    'SourceInputWidget',
    'SourceListInputWidget',
    'SourceMultiCheckBoxWidget',
    'SourceMultiSelectFrozenSetWidget',
    'SourceMultiSelectSetWidget',
    'SourceMultiSelectWidget',
    'SourceOrderedMultiSelectWidget',
    'SourceRadioWidget',
    'SourceSelectWidget',
    'SourceSequenceDisplayWidget',
]



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Form utility functions

In Zope 2's formulator, forms provide a basic mechanism for
organizing collections of fields and providing user interfaces for
them, especially editing interfaces.

In Zope 3, formulator's forms are replaced by Schema (See
zope.schema). In addition, the Formulator fields have been replaced by
schema fields and form widgets. Schema fields just express the semantics
of data values. They contain no presentation logic or parameters.
Widgets are views on fields that take care of presentation. The widget
view names represent styles that can be selected by applications to
customise the presentation. There can also be custom widgets with
specific parameters.

This module provides some utility functions that provide some of the
functionality of formulator forms that isn't handled by schema,
fields, or widgets.
"""
__docformat__ = 'restructuredtext'

from zope.formlib.interfaces import IDisplayWidget
from zope.formlib.interfaces import IInputWidget
from zope.formlib.interfaces import InputErrors
from zope.formlib.interfaces import MissingInputError
from zope.formlib.interfaces import WidgetsError
# BBB
from zope.formlib.utility import _fieldlist
from zope.formlib.utility import _widgetHasStickyValue
from zope.formlib.utility import applyWidgetsChanges
from zope.formlib.utility import no_value
from zope.formlib.utility import setUpWidget
from zope.formlib.utility import setUpWidgets
from zope.interface.interfaces import IMethod
from zope.proxy import isProxy
from zope.security.interfaces import ForbiddenAttribute
from zope.security.interfaces import Unauthorized
from zope.security.proxy import Proxy

from zope import security


def setUpEditWidgets(view, schema, source=None, prefix=None,
                     ignoreStickyValues=False, names=None, context=None,
                     degradeInput=False, degradeDisplay=False):
    """Sets up widgets to collect input on a view.

    See `setUpWidgets` for details on `view`, `schema`, `prefix`,
    `ignoreStickyValues`, `names`, and `context`.

    `source`, if specified, is an object from which initial widget values are
    read. If source is not specified, the view context is used as the source.

    `degradeInput` is a flag that changes the behavior when a user does not
    have permission to edit a field in the names.  By default, the function
    raises Unauthorized.  If degradeInput is True, the field is changed to
    an IDisplayWidget.

    `degradeDisplay` is a flag that changes the behavior when a user does not
    have permission to access a field in the names.  By default, the function
    raises Unauthorized.  If degradeDisplay is True, the field is removed from
    the form.

    Returns a list of names, equal to or a subset of the names that were
    supposed to be drawn, with uninitialized undrawn fields missing.
    """
    if context is None:
        context = view.context
    if source is None:
        source = view.context
    security_proxied = isProxy(source, Proxy)
    res_names = []
    for name, field in _fieldlist(names, schema):
        try:
            value = field.get(source)
        except ForbiddenAttribute:
            raise
        except AttributeError:
            value = no_value
        except Unauthorized:
            if degradeDisplay:
                continue
            else:
                raise
        if field.readonly:
            viewType = IDisplayWidget
        else:
            if security_proxied:
                is_accessor = IMethod.providedBy(field)
                if is_accessor:
                    set_name = field.writer.__name__
                    authorized = security.canAccess(source, set_name)
                else:
                    set_name = name
                    authorized = security.canWrite(source, name)
                if not authorized:
                    if degradeInput:
                        viewType = IDisplayWidget
                    else:
                        raise Unauthorized(set_name)
                else:
                    viewType = IInputWidget
            else:
                # if object is not security proxied, might be a standard
                # adapter without a registered checker.  If the feature of
                # paying attention to the users ability to actually set a
                # field is decided to be a must-have for the form machinery,
                # then we ought to change this case to have a deprecation
                # warning.
                viewType = IInputWidget
        setUpWidget(view, name, field, viewType, value, prefix,
                    ignoreStickyValues, context)
        res_names.append(name)
    return res_names


def setUpDisplayWidgets(view, schema, source=None, prefix=None,
                        ignoreStickyValues=False, names=None, context=None,
                        degradeDisplay=False):
    """Sets up widgets to display field values on a view.

    See `setUpWidgets` for details on `view`, `schema`, `prefix`,
    `ignoreStickyValues`, `names`, and `context`.

    `source`, if specified, is an object from which initial widget values are
    read. If source is not specified, the view context is used as the source.

    `degradeDisplay` is a flag that changes the behavior when a user does not
    have permission to access a field in the names.  By default, the function
    raises Unauthorized.  If degradeDisplay is True, the field is removed from
    the form.

    Returns a list of names, equal to or a subset of the names that were
    supposed to be drawn, with uninitialized undrawn fields missing.
    """
    if context is None:
        context = view.context
    if source is None:
        source = view.context
    res_names = []
    for name, field in _fieldlist(names, schema):
        try:
            value = field.get(source)
        except ForbiddenAttribute:
            raise
        except AttributeError:
            value = no_value
        except Unauthorized:
            if degradeDisplay:
                continue
            else:
                raise
        setUpWidget(view, name, field, IDisplayWidget, value, prefix,
                    ignoreStickyValues, context)
        res_names.append(name)
    return res_names


def viewHasInput(view, schema, names=None):
    """Returns ``True`` if the any of the view's widgets contain user input.

    `schema` specifies the set of fields that correspond to the view widgets.

    `names` can be specified to provide a subset of these fields.
    """
    for name, field in _fieldlist(names, schema):
        if getattr(view, name + '_widget').hasInput():
            return True
    return False


def getWidgetsData(view, schema, names=None):
    """Returns user entered data for a set of `schema` fields.

    The return value is a map of field names to data values.

    `view` is the view containing the widgets. `schema` is the schema that
    defines the widget fields. An optional `names` argument can be provided
    to specify an alternate list of field values to return. If `names` is
    not specified, or is ``None``, `getWidgetsData` will attempt to return
    values for all of the fields in the schema.

    A requested field value may be omitted from the result for one of two
    reasons:

        - The field is read only, in which case its widget will not have
          user input.

        - The field is editable and not required but its widget does not
          contain user input.

    If a field is required and its widget does not have input, `getWidgetsData`
    raises an error.

    A widget may raise a validation error if it cannot return a value that
    satisfies its field's contraints.

    Errors, if any, are collected for all fields and reraised as a single
    `WidgetsError`.
    """
    result = {}
    errors = []

    for name, field in _fieldlist(names, schema):
        widget = getattr(view, name + '_widget')
        if IInputWidget.providedBy(widget):
            if widget.hasInput():
                try:
                    result[name] = widget.getInputValue()
                except InputErrors as error:
                    errors.append(error)
            elif field.required:
                errors.append(MissingInputError(
                    name, widget.label, 'the field is required'))

    if errors:
        raise WidgetsError(errors, widgetsData=result)

    return result


__all__ = [
    # BBB
    '_widgetHasStickyValue',
    'applyWidgetsChanges',
    'setUpWidget',
    'setUpWidgets',
    # From this module
    'getWidgetsData',
    'setUpDisplayWidgets',
    'setUpEditWidgets',
    'viewHasInput',
]



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Editview tests"""
import unittest

import transaction
from persistent import Persistent
from webtest import TestApp
from zope.interface import Interface
from zope.interface import implementer
from zope.schema import TextLine
from zope.traversing.api import traverse

from zope.app.form.browser.editview import EditView
from zope.app.form.browser.tests.support import defineSecurity
from zope.app.form.browser.tests.support import patternExists
from zope.app.form.browser.tests.support import registerEditForm
from zope.app.form.testing import AppFormLayer


class IFoo(Interface):

    optional_text = TextLine(required=False)
    required_text = TextLine(required=True)


@implementer(IFoo)
class Foo(Persistent):
    pass


class Test(unittest.TestCase):

    layer = AppFormLayer

    def setUp(self):
        self.getRootFolder = self.layer.getRootFolder
        registerEditForm(IFoo)
        defineSecurity(Foo, IFoo)
        self._testapp = TestApp(self.layer.make_wsgi_app())

    def publish(self, path, basic=None, form=None, headers=None):
        if basic:
            self._testapp.authorization = ('Basic', tuple(basic.split(':')))
        else:
            self._testapp.authorization = None
        env = {'wsgi.handleErrors': False}
        if form:
            response = self._testapp.post(path, params=form,
                                          extra_environ=env, headers=headers)
        else:
            response = self._testapp.get(
                path, extra_environ=env, headers=headers)
        return response

    def test_rollback_on_error(self):
        """Tests rollback when a widget error occurs.

        When one or more errors are generated by input widgets, the current
        transaction should be rolledback to ensure object integrity.
        """
        self.getRootFolder()['foo'] = Foo()
        self.getRootFolder()['foo'].required_text = 'initial required'
        self.getRootFolder()['foo'].optional_text = 'initial optional'
        transaction.commit()

        # submit form with legal value for optional_text and invalid for
        # required_text
        old_update = EditView.update
        try:
            def new_update(self):
                # This update changes something after form validation has
                # failed. Side effects like this should not be committed.
                # http://www.zope.org/Collectors/Zope3-dev/655
                result = old_update(self)
                self.context.required_text = 'changed after form validation'
                return result
            EditView.update = new_update
            response = self.publish('/foo/edit.html', form={
                'field.optional_text': '',
                'field.required_text': '',
                'UPDATE_SUBMIT': ''})
            self.assertEqual(response.status_int, 200)
        finally:
            EditView.update = old_update

        # confirm that one errors exists
        self.assertTrue(patternExists(
            'There are <strong>1</strong> input errors.', response.text))

        # confirm that foo was not modified
        foo = traverse(self.getRootFolder(), 'foo')
        self.assertEqual(foo.required_text, 'initial required')
        self.assertEqual(foo.optional_text, 'initial optional')


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser widgets for items"""
__docformat__ = 'restructuredtext'

# BBB the implementation has moved to zope.formlib.itemswidgets
from zope.formlib.itemswidgets import EXPLICIT_EMPTY_SELECTION
from zope.formlib.itemswidgets import ChoiceCollectionDisplayWidget
from zope.formlib.itemswidgets import ChoiceCollectionInputWidget
from zope.formlib.itemswidgets import ChoiceDisplayWidget
from zope.formlib.itemswidgets import ChoiceInputWidget
from zope.formlib.itemswidgets import CollectionDisplayWidget
from zope.formlib.itemswidgets import CollectionInputWidget
from zope.formlib.itemswidgets import DropdownWidget
from zope.formlib.itemswidgets import ItemDisplayWidget
from zope.formlib.itemswidgets import ItemsEditWidgetBase
from zope.formlib.itemswidgets import ItemsMultiDisplayWidget
from zope.formlib.itemswidgets import ItemsMultiEditWidgetBase
from zope.formlib.itemswidgets import ItemsWidgetBase
from zope.formlib.itemswidgets import ListDisplayWidget
from zope.formlib.itemswidgets import MultiCheckBoxWidget
from zope.formlib.itemswidgets import MultiDataHelper
from zope.formlib.itemswidgets import MultiSelectFrozenSetWidget
from zope.formlib.itemswidgets import MultiSelectSetWidget
from zope.formlib.itemswidgets import MultiSelectWidget
from zope.formlib.itemswidgets import OrderedMultiSelectWidget
from zope.formlib.itemswidgets import RadioWidget
from zope.formlib.itemswidgets import SelectWidget
from zope.formlib.itemswidgets import SetDisplayWidget
from zope.formlib.itemswidgets import SingleDataHelper
from zope.formlib.itemswidgets import TranslationHook


__all__ = [
    'ChoiceCollectionDisplayWidget',
    'ChoiceCollectionInputWidget',
    'ChoiceDisplayWidget',
    'ChoiceInputWidget',
    'CollectionDisplayWidget',
    'CollectionInputWidget',
    'DropdownWidget',
    'EXPLICIT_EMPTY_SELECTION',
    'ItemDisplayWidget',
    'ItemsEditWidgetBase',
    'ItemsMultiDisplayWidget',
    'ItemsMultiEditWidgetBase',
    'ItemsWidgetBase',
    'ListDisplayWidget',
    'MultiCheckBoxWidget',
    'MultiDataHelper',
    'MultiSelectFrozenSetWidget',
    'MultiSelectSetWidget',
    'MultiSelectWidget',
    'OrderedMultiSelectWidget',
    'RadioWidget',
    'SelectWidget',
    'SetDisplayWidget',
    'SingleDataHelper',
    'TranslationHook',
]



##############################################################################
#
# Copyright (c) 2001-2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser Widget Definitions"""
__docformat__ = 'restructuredtext'

# BBB
from zope.formlib.widget import BrowserWidget
from zope.formlib.widget import DisplayWidget
from zope.formlib.widget import SimpleInputWidget
from zope.formlib.widget import UnicodeDisplayWidget
from zope.formlib.widget import escape
from zope.formlib.widget import quoteattr
from zope.formlib.widget import renderElement
from zope.formlib.widget import renderTag
from zope.formlib.widget import setUp
from zope.formlib.widget import tearDown


__all__ = [
    'quoteattr',
    'BrowserWidget',
    'SimpleInputWidget',
    'DisplayWidget',
    'UnicodeDisplayWidget',
    'renderTag',
    'renderElement',
    'escape',
    'setUp',
    'tearDown',
]



#############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Form Directives Tests"""
import unittest
from io import StringIO

from zope.component.testing import PlacelessSetup
from zope.configuration.xmlconfig import XMLConfig
from zope.configuration.xmlconfig import xmlconfig
from zope.interface import Interface
from zope.interface import implementer
from zope.publisher.browser import TestRequest
from zope.schema import Int
from zope.schema import TextLine
from zope.traversing.interfaces import TraversalError

from zope import component
from zope.app.form.browser import TextWidget
from zope.app.form.tests import utils


template = """<configure
   xmlns='http://namespaces.zope.org/zope'
   xmlns:browser='http://namespaces.zope.org/browser'
   i18n_domain='zope'>
   %s
   </configure>"""


request = TestRequest()


class Schema(Interface):

    text = TextLine(
        title='Text',
        description='Nice text',
        required=False)


class IC(Schema):
    pass


@implementer(IC)
class Ob:
    pass


unwrapped_ob = Ob()
ob = utils.securityWrap(unwrapped_ob, IC)


class ISomeWidget(Interface):
    displayWidth = Int(
        title="Display Width",
        default=20,
        required=True)


@implementer(ISomeWidget)
class SomeWidget(TextWidget):
    pass


class Test(PlacelessSetup, unittest.TestCase):

    def setUp(self):
        super().setUp()
        import zope.component
        XMLConfig('meta.zcml', zope.component)()
        import zope.app.form.browser
        XMLConfig('meta.zcml', zope.app.form.browser)()
        import zope.browsermenu
        XMLConfig('meta.zcml', zope.browsermenu)()

        from zope.traversing.adapters import DefaultTraversable
        from zope.traversing.interfaces import ITraversable

        component.provideAdapter(DefaultTraversable, (None,), ITraversable)

    def testAddForm(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, request), name='add.html'),
            None)
        xmlconfig(StringIO(template % ("""
          <view
              type="zope.publisher.interfaces.browser.IBrowserRequest"
              for="zope.schema.interfaces.ITextLine"
              provides="zope.formlib.interfaces.IInputWidget"
              factory="zope.app.form.browser.TextWidget"
              permission="zope.Public"
              />

          <browser:addform
              for="zope.app.form.browser.tests.test_directives.IC"
              schema="zope.app.form.browser.tests.test_directives.Schema"
              name="add.html"
              label="Add a ZPT page"
              fields="text"
              permission="zope.Public" />
            """)))

        v = component.getMultiAdapter((ob, request), name='add.html')
        # expect to fail as standard macros are not configured
        self.assertRaises(TraversalError, v)

    def testEditForm(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, request), name='edit.html'),
            None)
        xmlconfig(StringIO(template % ("""
          <view
              type="zope.publisher.interfaces.browser.IBrowserRequest"
              for="zope.schema.interfaces.ITextLine"
              provides="zope.formlib.interfaces.IInputWidget"
              factory="zope.app.form.browser.TextWidget"
              permission="zope.Public"
              />

          <browser:editform
              for="zope.app.form.browser.tests.test_directives.IC"
              schema="zope.app.form.browser.tests.test_directives.Schema"
              name="edit.html"
              label="Edit a ZPT page"
              fields="text"
              permission="zope.Public" />
            """)))

        v = component.getMultiAdapter((ob, request), name='edit.html')
        # expect to fail as standard macros are not configured
        self.assertRaises(TraversalError, v)

    def testEditFormWithMenu(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, request), name='edit.html'),
            None)
        xmlconfig(StringIO(template % ('''
          <browser:menu id="test_menu" title="Test menu"/>
          <view
              type="zope.publisher.interfaces.browser.IBrowserRequest"
              for="zope.schema.interfaces.ITextLine"
              provides="zope.formlib.interfaces.IInputWidget"
              factory="zope.app.form.browser.TextWidget"
              permission="zope.Public"
              />
          <browser:editform
              for="zope.app.form.browser.tests.test_directives.IC"
              schema="zope.app.form.browser.tests.test_directives.Schema"
              name="edit.html"
              label="Edit a ZPT page"
              fields="text"
              permission="zope.Public"
              menu="test_menu"
              title="Test View"
              />
            ''')))

        v = component.queryMultiAdapter((ob, request), name='edit.html')
        # expect to fail as standard macros are not configured
        self.assertRaises(TraversalError, v)

    def testSchemaDisplay(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, request), name='view.html'),
            None)
        xmlconfig(StringIO(template % ('''
          <view
              type="zope.publisher.interfaces.browser.IBrowserRequest"
              for="zope.schema.interfaces.IField"
              provides="zope.formlib.interfaces.IDisplayWidget"
              factory="zope.app.form.browser.DisplayWidget"
              permission="zope.Public"
              />

          <browser:schemadisplay
              for="zope.app.form.browser.tests.test_directives.IC"
              schema="zope.app.form.browser.tests.test_directives.Schema"
              name="view.html"
              label="View a ZPT page"
              fields="text"
              permission="zope.Public" />
            ''')))

        v = component.queryMultiAdapter((ob, request), name='view.html')
        # expect to fail as standard macros are not configured
        self.assertRaises(TraversalError, v)

    def testAddFormWithWidget(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, request), name='add.html'),
            None)
        xmlconfig(StringIO(template % ('''
          <view
              type="zope.publisher.interfaces.browser.IBrowserRequest"
              for="zope.schema.interfaces.ITextLine"
              provides="zope.formlib.interfaces.IInputWidget"
              factory="zope.app.form.browser.TextWidget"
              permission="zope.Public"
              />

          <browser:addform
              for="zope.app.form.browser.tests.test_directives.IC"
              schema="zope.app.form.browser.tests.test_directives.Schema"
              name="add.html"
              label="Add a ZPT page"
              fields="text"
              permission="zope.Public">

            <widget
                field="text"
                class="zope.app.form.browser.tests.test_directives.SomeWidget"
                displayWidth="30"
                extra="foo"
                />

          </browser:addform>
            ''')), )

        view = component.queryMultiAdapter((ob, request), name='add.html')
        self.assertTrue(hasattr(view, 'text_widget'))
        self.assertIsInstance(view.text_widget, SomeWidget)
        self.assertEqual(view.text_widget.extra, 'foo')
        self.assertEqual(view.text_widget.displayWidth, 30)

    def testEditFormWithWidget(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, request), name='edit.html'),
            None)
        xmlconfig(StringIO(template % ('''
          <view
              type="zope.publisher.interfaces.browser.IBrowserRequest"
              for="zope.schema.interfaces.ITextLine"
              provides="zope.formlib.interfaces.IInputWidget"
              factory="zope.app.form.browser.TextWidget"
              permission="zope.Public"
              />

          <browser:editform
              for="zope.app.form.browser.tests.test_directives.IC"
              schema="zope.app.form.browser.tests.test_directives.Schema"
              name="edit.html"
              label="Edit a ZPT page"
              fields="text"
              permission="zope.Public">

            <widget
                field="text"
                class="zope.app.form.browser.tests.test_directives.SomeWidget"
                displayWidth="30"
                extra="foo"
                />

          </browser:editform>
            ''')), )

        view = component.queryMultiAdapter((ob, request), name='edit.html')
        self.assertTrue(hasattr(view, 'text_widget'))
        self.assertIsInstance(view.text_widget, SomeWidget)
        self.assertEqual(view.text_widget.extra, 'foo')
        self.assertEqual(view.text_widget.displayWidth, 30)

    def testSchemaDisplayWithWidget(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, request), name='view.html'),
            None)
        xmlconfig(StringIO(template % ('''
          <view
              type="zope.publisher.interfaces.browser.IBrowserRequest"
              for="zope.schema.interfaces.IField"
              provides="zope.formlib.interfaces.IDisplayWidget"
              factory="zope.app.form.browser.DisplayWidget"
              permission="zope.Public"
              />

          <browser:schemadisplay
              for="zope.app.form.browser.tests.test_directives.IC"
              schema="zope.app.form.browser.tests.test_directives.Schema"
              name="view.html"
              label="View a ZPT page"
              fields="text"
              permission="zope.Public">

            <browser:widget
                field="text"
                class="zope.app.form.browser.tests.test_directives.SomeWidget"
                displayWidth="30"
                extra="foo"
                />
          </browser:schemadisplay>
            ''')))

        view = component.queryMultiAdapter((ob, request), name='view.html')
        self.assertTrue(hasattr(view, 'text_widget'))
        self.assertIsInstance(view.text_widget, SomeWidget)
        self.assertEqual(view.text_widget.extra, 'foo')
        self.assertEqual(view.text_widget.displayWidth, 30)


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# This package is developed by the Zope Toolkit project, documented here:
# https://zopetoolkit.readthedocs.io/
# When developing and releasing this package, please follow the documented
# Zope Toolkit policies as described by this documentation.
##############################################################################
"""Setup for zope.app.form package
"""
import os

from setuptools import setup


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


tests_require = [
    'zc.sourcefactory',
    'zope.container',
    'zope.principalregistry',
    'zope.site',
    'zope.traversing',
    'zope.app.appsetup',
    'zope.testing',
    'zope.testrunner >= 6.4',
    'zope.app.wsgi >= 5.3',
    'webtest',
]

setup(name='zope.app.form',
      version='7.0',
      author='Zope Foundation and Contributors',
      author_email='zope-dev@zope.dev',
      description='The Original Zope 3 Form Framework',
      long_description=(
          read('README.rst')
          + '\n\n' +
          'Detailed documentation:\n'
          + '\n\n' +
          read('CHANGES.rst')
      ),
      keywords="zope3 form widget zcml",
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Zope Public License',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.9',
          'Programming Language :: Python :: 3.10',
          'Programming Language :: Python :: 3.11',
          'Programming Language :: Python :: 3.12',
          'Programming Language :: Python :: 3.13',
          'Programming Language :: Python :: Implementation :: CPython',
          'Programming Language :: Python :: Implementation :: PyPy',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Topic :: Internet :: WWW/HTTP',
          'Framework :: Zope :: 3',
      ],
      url='https://github.com/zopefoundation/zope.app.form',
      license='ZPL-2.1',
      python_requires='>=3.9',
      extras_require={
          'test': tests_require
      },
      install_requires=[
          "setuptools",
          "transaction",
          "zope.formlib >= 4.3",
          "zope.browser >= 2.1",
          "zope.browserpage >= 3.10.1",
          "zope.browsermenu",
          "zope.component",
          "zope.configuration >= 4.4.0",
          "zope.datetime",
          "zope.exceptions",
          "zope.i18n",
          "zope.interface",
          "zope.proxy",
          "zope.publisher >= 4.3.1",
          "zope.schema >= 4.4.2",
          "zope.security",
      ],
      include_package_data=True,
      zip_safe=False,
      )
